<?php
/*
Plugin Name: Now Reading
Version: 2.1
Plugin URI: http://robm.me.uk/projects/plugins/wordpress/now-reading/
Description: Allows you to display the books you're reading, have read recently and plan to read, with cover art fetched automatically from Amazon.
Author: Rob Miller
Author URI: http://robm.me.uk/
*/
/*
Note: this is an adaptation of the official 2.1 version, containing these extra features:
- review field
- favorites category
- nr_get_book accepts list of categories to display
URI: http://www.blog.dannynet.net/archives/17
*/

// Create our table, if it doesn't exist already.
$wpdb->query('
CREATE TABLE IF NOT EXISTS `'.$table_prefix.'now_reading` (
`b_id` BIGINT NOT NULL AUTO_INCREMENT ,
`b_added` DATETIME NOT NULL ,
`b_title` TEXT NOT NULL ,
`b_author` TEXT NOT NULL ,
`b_image` TEXT NOT NULL ,
`b_asin` VARCHAR(255) NOT NULL,
PRIMARY KEY ( `b_id` )
) TYPE = MYISAM ;
');

// Check for old versions that don't have new fields.
// Rather hackish solution, but SQL doesn't have anything
// along the lines of "ALTER TABLE IF NOT EXISTS fieldname ...",
// which is annoying.
$columns = $wpdb->get_results('SHOW COLUMNS FROM `'.$table_prefix.'now_reading`');

$asin_exists = false;
$status_exists = false;
$finished_exists = false;
$review_exists = false;

foreach($columns as $column) {
	if($column->Field == 'b_asin') {
		$asin_exists = true;
	}
	if($column->Field == 'b_status') {
		$status_exists = true;
	}
	if($column->Field == 'b_finished') {
		$finished_exists = true;
	}
	if($column->Field == 'b_review') {
		$review_exists = true;
	}
}
if(!$asin_exists) {
	$wpdb->query('
	ALTER TABLE `'.$table_prefix.'now_reading`
	ADD `b_asin` VARCHAR(255) NOT NULL
	');
}
if(!$status_exists) {
	$wpdb->query('
	ALTER TABLE `'.$table_prefix.'now_reading`
	ADD `b_status` ENUM("read", "reading", "unread") NOT NULL
	');
} else {
	$wpdb->query('
	ALTER TABLE `'.$table_prefix.'now_reading`
	MODIFY `b_status` ENUM("read", "reading", "unread", "favorites") NOT NULL
	');
}

if(!$finished_exists) {
	$wpdb->query('
	ALTER TABLE `'.$table_prefix.'now_reading`
	ADD `b_finished` DATETIME NOT NULL AFTER `b_added`
	');
}

if(!$review_exists) {
	$wpdb->query('
	ALTER TABLE `'.$table_prefix.'now_reading`
	ADD `b_review` TEXT AFTER `b_added`
	');
}

// Set up default options
add_option('nr_current_format_string', '
<li>
	<p><a href="%link%"><img src="%image%" alt="" id="nr_image" /></a></p>
	<p><cite>%title%</cite> by <strong>%author%</strong></p>
</li>
');
add_option('nr_read_format_string', '
<li>
	<a href="%link%"><cite>%title%</cite></a> by <strong>%author%</strong>
</li>
');
add_option('nr_unread_format_string', '
<li>
	<a href="%link%"><cite>%title%</cite></a> by <strong>%author%</strong>
</li>
');
add_option('nr_favorites_format_string', '
<li>
	<a href="%link%"><cite>%title%</cite></a> by <strong>%author%</strong>
</li>
');

add_option('nr_num_current', 3);
add_option('nr_num_unread', 5);
add_option('nr_num_read', 5);
add_option('nr_num_favorites', 5);

add_option('nr_current_outer_tag', 'ul');
add_option('nr_unread_outer_tag', 'ul');
add_option('nr_read_outer_tag', 'ul');
add_option('nr_favorites_outer_tag', 'ul');

add_option('nr_current_text', '<p>Current books:</p>');
add_option('nr_unread_text', '<p>Planned books:</p>');
add_option('nr_read_text', '<p>Recent books:</p>');
add_option('nr_favorites_text', '<p>Favorite books:</p>');

add_option('nr_format_date', '%D %b %Y');
add_option('nr_associate', 'roblog-21');
add_option('nr_domain', '.com');

// Add the various sub-menus of the admin panel.
function nr_add_pages() {

	add_management_page('Now Reading', 'Now Reading', 9, basename(__FILE__), 'nr_manage');
	add_submenu_page('post.php', 'Now Reading', 'Now Reading', 9, basename(__FILE__), 'now_reading');
	add_options_page('Now Reading', 'Now Reading', 9, basename(__FILE__), 'nr_options');

}

// Manages the options sub-menu
function nr_options() {

	foreach($_POST as $key => $val) {
		$_POST[$key] = stripslashes($val);
	}

	global $wpdb, $table_prefix;

	if(!empty($_POST['update'])) {

		update_option('nr_current_format_string', $_POST['current_format_string']);
		update_option('nr_read_format_string', $_POST['read_format_string']);
		update_option('nr_unread_format_string', $_POST['unread_format_string']);
		update_option('nr_favorites_format_string', $_POST['favorites_format_string']);

		update_option('nr_num_current', intval($_POST['num_current']));
		update_option('nr_num_unread', intval($_POST['num_unread']));
		update_option('nr_num_read', intval($_POST['num_read']));
		update_option('nr_num_favorites', intval($_POST['num_favorites']));

		update_option('nr_current_text', $_POST['current_text']);
		update_option('nr_unread_text', $_POST['unread_text']);
		update_option('nr_read_text', $_POST['read_text']);
		update_option('nr_favorites_text', $_POST['favorites_text']);

		update_option('nr_current_outer_tag', $_POST['current_outer_tag']);
		update_option('nr_unread_outer_tag', $_POST['unread_outer_tag']);
		update_option('nr_read_outer_tag', $_POST['read_outer_tag']);
		update_option('nr_favorites_outer_tag', $_POST['favorites_outer_tag']);

		update_option('nr_current_text', $_POST['current_text']);
		update_option('nr_unread_text', $_POST['unread_text']);
		update_option('nr_read_text', $_POST['read_text']);
		update_option('nr_favorites_text', $_POST['favorites_text']);

		update_option('nr_format_date', $_POST['format_date']);
		update_option('nr_associate', $_POST['associate']);
		update_option('nr_domain', $_POST['domain']);

		echo '
		<div class="updated">
			<p><strong>Options saved.</strong></p>
		</div>
		';

	}

	echo '
	<div class="wrap">

		<h2>Now Reading</h2>
	';


	$current_format_string = get_option('nr_current_format_string');
	$read_format_string = get_option('nr_read_format_string');
	$unread_format_string = get_option('nr_unread_format_string');
	$favorites_format_string = get_option('nr_favorites_format_string');

	$num_current = get_option('nr_num_current');
	$num_unread = get_option('nr_num_unread');
	$num_read = get_option('nr_num_read');
	$num_favorites = get_option('nr_num_favorites');

	$current_outer_tag = get_option('nr_current_outer_tag');
	$unread_outer_tag = get_option('nr_unread_outer_tag');
	$read_outer_tag = get_option('nr_read_outer_tag');
	$favorites_outer_tag = get_option('nr_favorites_outer_tag');

	$current_text = get_option('nr_current_text');
	$unread_text = get_option('nr_unread_text');
	$read_text = get_option('nr_read_text');
	$favorites_text = get_option('nr_favorites_text');

	$format_date = get_option('nr_format_date');
	$associate = get_option('nr_associate');
	$domain = get_option('nr_domain');

	echo '
		<form method="post" action="'.$_SERVER['REQUEST_URI'].'">

		<fieldset>

			<legend>Books you\'re current reading</legend>

			<p>This section deals with books that have a status of "reading".</p>

			<table width="100%" cellspacing="2" cellpadding="5">
				<tr valign="top">
					<th width="33%" scope="row">Format string:</th>
					<td>
						<textarea name="current_format_string" id="current_format_string" rows="3" cols="60">'.htmlentities($current_format_string).'</textarea>
						<br />
						How to display books that you\'re currently reading. Acceptable variables are:
						<ul>
							<li>%image% - the URL to the cover image.</li>
							<li>%title% - the book\'s title</li>
							<li>%author% - the book\'s author</li>
							<li>%date% - the date the book was added</li>
							<li>%link% - the URL to the product page on Amazon.com, including your Associates ID.</li>
							<li>%review% - a short description or review of the book, or link to a post.</li>
						</ul>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row">Outer element:</th>
					<td>
						<input type="text" name="current_outer_tag" value="'.htmlentities($current_outer_tag).'" />
						<br />
						The outer tag that comes before and after the loop of items. If you have each item as a &lt;li&gt;, then this should be set to <code>ul</code>, for example.
					</td>
				</tr>
			<tr valign="top">
				<th width="33%" scope="row">Text:</th>
				<td>
					<input type="text" name="current_text" value="'.htmlentities($current_text).'" />
					<br />
					The text that is displayed before the list of books.
				</td>
			</tr>
				<tr valign="top">
					<th width="33%" scope="row">Books to display:</th>
					<td>
						<input type="text" name="num_current" value="'.intval($num_current).'" />
						<br />
						The maximum number of current books that will be displayed. Entering 0 will also not display the text and outer element.
					</td>
				</tr>
			</table>

	</fieldset>

	<fieldset>

		<legend>Books you\'ve yet to read</legend>

		<p>This section deals with books that have a status of "unread".</p>

		<table width="100%" cellspacing="2" cellpadding="5">
			<tr valign="top">
				<th width="33%" scope="row">Format string:</th>
				<td>
					<textarea name="unread_format_string" id="unread_format_string" rows="3" cols="60">'.htmlentities($unread_format_string).'</textarea>
					<br />
					How to display books that you have yet to read. Variables are the same as the current format string.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Outer element:</th>
				<td>
					<input type="text" name="unread_outer_tag" value="'.htmlentities($unread_outer_tag).'" />
					<br />
					The outer tag that comes before and after the loop of items. If you have each item as a &lt;li&gt;, then this should be set to <code>ul</code>, for example.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Text:</th>
				<td>
					<input type="text" name="unread_text" value="'.htmlentities($unread_text).'" />
					<br />
					The text that is displayed before the list of books.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Books to display:</th>
				<td>
					<input type="text" name="num_unread" value="'.intval($num_unread).'" />
					<br />
					The maximum number of current books that will be displayed. Entering 0 will also not display the text and outer element.
				</td>
			</tr>
		</table>

	</fieldset>

	<fieldset>

		<legend>Books you\'ve recently finished</legend>

		<p>This section deals with books that have a status of "read".</p>

		<table width="100%" cellspacing="2" cellpadding="5">
			<tr valign="top">
				<th width="33%" scope="row">Format string:</th>
				<td>
					<textarea name="read_format_string" id="read_format_string" rows="3" cols="60">'.htmlentities($read_format_string).'</textarea>
					<br />
					How to display books that have already been read. Variables are the same as the current format string.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Outer element:</th>
				<td>
					<input type="text" name="read_outer_tag" value="'.htmlentities($read_outer_tag).'" />
					<br />
					The outer tag that comes before and after the loop of items. If you have each item as a &lt;li&gt;, then this should be set to <code>ul</code>, for example.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Text:</th>
				<td>
					<input type="text" name="read_text" value="'.htmlentities($read_text).'" />
					<br />
					The text that is displayed before the list of books.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Books to display:</th>
				<td>
					<input type="text" name="num_read" value="'.intval($num_read).'" />
					<br />
					The maximum number of current books that will be displayed. Entering 0 will also not display the text and outer element.
				</td>
			</tr>
		</table>

	</fieldset>

	<fieldset>

		<legend>Your all time favorite books</legend>

		<p>This section deals with books that have a status of "favorite".</p>

		<table width="100%" cellspacing="2" cellpadding="5">
			<tr valign="top">
				<th width="33%" scope="row">Format string:</th>
				<td>
					<textarea name="favorites_format_string" id="favorites_format_string" rows="3" cols="60">'.htmlentities($favorites_format_string).'</textarea>
					<br />
					How to display favorite books. Variables are the same as the current format string.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Outer element:</th>
				<td>
					<input type="text" name="favorites_outer_tag" value="'.htmlentities($favorites_outer_tag).'" />
					<br />
					The outer tag that comes before and after the loop of items. If you have each item as a &lt;li&gt;, then this should be set to <code>ul</code>, for example.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Text:</th>
				<td>
					<input type="text" name="favorites_text" value="'.htmlentities($favorites_text).'" />
					<br />
					The text that is displayed before the list of books.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Books to display:</th>
				<td>
					<input type="text" name="num_favorites" value="'.intval($num_favorites).'" />
					<br />
					The maximum number of favorite books that will be displayed. Entering 0 will also not display the text and outer element.
				</td>
			</tr>
		</table>

	</fieldset>

	<fieldset>

		<legend>General Options</legend>

		<table width="100%" cellspacing="2" cellpadding="5">
			<tr valign="top">
				<th width="33%" scope="row">Date format string:</th>
				<td>
					<input type="text" name="format_date" value="'.htmlentities($format_date).'" />
					<br />
					How to format the date if it\'s displayed in a format string. Acceptable variables can be found <a href="http://dev.mysql.com/doc/mysql/en/date-and-time-functions.html#id3101576">here</a>.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Your Amazon Associates ID:</th>
				<td>
					<input type="text" name="associate" value="'.htmlentities($associate).'" />
					<br />
					If you choose to link to your book\'s product page on Amazon.com using the <code>%link%</code> variable, you can earn commission if your visitors then purchase products. If you don\'t have an Amazon Associates ID, you can either <a href="http://associates.amazon.com">get one</a>, or consider entering <strong>roblog-21</strong> if you\'re feeling generous.
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row">Amazon domain to link to:</th>
				<td>
					<select name="domain">
						<option'.(($domain == '.com') ? ' selected="selected"' : '').' value=".com">Amazon.com</option>
						<option'.(($domain == '.co.uk') ? ' selected="selected"' : '').' value=".co.uk">Amazon.co.uk</option>
						<option'.(($domain == '.co.jp') ? ' selected="selected"' : '').' value=".co.jp">Amazon.co.jp</option>
						<option'.(($domain == '.de') ? ' selected="selected"' : '').' value=".de">Amazon.de</option>
					<br />
					If you choose to link to your book\'s product page on Amazon.com using the <code>%link%</code> variable, you can specify which country-specific Amazon site to link to.
				</td>
			</tr>
		</table>

	</fieldset>

		<input type="hidden" name="update" value="yes" />

		<p class="submit">
			<input type="submit" value="Update Options &raquo;" />
		</p>

		</form>

	</div>
	';

}

// The Manage sub-menu
function nr_manage() {

	global $wpdb, $table_prefix;

	foreach($_POST as $key => $val) {
		$_POST[$key] = stripslashes($val);
	}

	if($_GET['action'] == 'delete') {

		$id = intval($_GET['id']);

		$wpdb->query('
		DELETE FROM '.$table_prefix.'now_reading
		WHERE b_id = '.$id.'
		');

		echo '

			<div class="updated">
				<p><strong>Book deleted successfully.</strong></p>
			</div>

		';

	}

	elseif($_POST['action'] == 'update') {

		$id = intval($_POST['id']);
		$author = mysql_real_escape_string($_POST['author']);
		$title = mysql_real_escape_string($_POST['title']);
		$status = $_POST['status'];
		$added = $_POST['added'];
		$review = mysql_real_escape_string($_POST['review']);

		$wpdb->query("
		UPDATE {$table_prefix}now_reading
		SET
			b_author = '$author',
			b_title = '$title',
			b_status = '$status',
			b_added = '$added',
			b_review = '$review'
		WHERE
			b_id = $id
		");

		echo '
		<div class="updated">
			<p><strong>Book updated.</strong></p>
		</div>
		';

	}

	$count = $wpdb->get_row('
	SELECT COUNT(*) AS count
	FROM '.$table_prefix.'now_reading
	');

	if($count->count) {

		$books = $wpdb->get_results('
		SELECT b_id AS id, b_title AS title, b_author AS author, b_added AS added, b_status AS status, b_image AS image, b_review as review
		FROM '.$table_prefix.'now_reading
		ORDER BY b_status DESC, b_added DESC
		');

		echo '

		<div class="wrap">

			<h2>Now Reading</h2>

			<table width="100%" cellpadding="3" cellspacing="3">

				<tr>
					<th scope="col"></th>
					<th scope="col">ID</th>
					<th scope="col">Title</th>
					<th scope="col">Author</th>
					<th scope="col">Status</th>
					<th scope="col">Date Added</th>
					<th scope="col"></th>
					<th scope="col"></th>
				</tr>
		';

		$i = 0;

		foreach($books as $book) {

			$alt = ($i % 2 == 0) ? ' class="alternate"' : '';
			echo '
			<form method="post" action="'.$_SERVER['REQUEST_URI'].'">
				<input type="hidden" name="action" value="update" />
				<tr'.$alt.'>
					<td><img src="'.$book->image.'" width="45px" height="70px" /></td>
					<td><input type="hidden" name="id" value="'.$book->id.'" />'.$book->id.'</td>
					<td><input type="text" name="title" value="'.$book->title.'" style="width:90%" /></td>
					<td><input type="text" name="author" value="'.$book->author.'" style="width:90%" /></td>
					<td>
						<select name="status">
							<option value="unread"'.(($book->status == 'unread') ? ' selected="selected"' : '').'>Yet to read</option>
							<option value="reading"'.(($book->status == 'reading') ? ' selected="selected"' : '').'>Currently reading</option>
							<option value="read"'.(($book->status == 'read') ? ' selected="selected"' : '').'>Finished</option>
							<option value="favorites"'.(($book->status == 'favorites') ? ' selected="selected"' : '').'>Favorite</option>
						</select>
					</td>
					<td><input type="text" name="added" value="'.$book->added.'" style="width:90%" /></td>
					<td><input type="submit" value="Update" /></td>
					<td><a href="'.$_SERVER['REQUEST_URI'].'&action=delete&id='.$book->id.'" onclick="return confirm(\'Are you sure you wish to delete this book permanently?\');">Delete</a></td>
				</tr>
				<tr'.$alt.'>
					<td>Review</td>
					<td colspan="7"><textarea name="review" style="width:90%">'.$book->review.'</textarea></td>
				</tr>
			</form>
			';

			$i++;

		}

		echo '
			</table>
		';

	} else {

		echo 'No books to display.';

	}

	echo '
	</div>
	';

}

// The write submenu
function now_reading() {


	foreach($_POST as $key => $val) {
		$_POST[$key] = stripslashes($val);
	}

	global $wpdb, $table_prefix;

	echo '
	<div class="wrap">

		<h2>Now Reading</h2>
	';

	if(!empty($_POST['u_title'])) {

		if(!class_exists('snoopy')) {
			require_once 'snoopy.php';
		}
		$snoopy = new snoopy;

		$u_title = $_POST['u_title'];
		$u_author = $_POST['u_author'];

		// Build the search URL.
		$url = 'http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&SubscriptionId=04FAYTZEMJE0WMQSPMR2&Operation=ItemSearch&SearchIndex=Books&ResponseGroup=Request,Small,Images&Version=2004-11-10&Title='.urlencode($u_title);

		if(!empty($u_author)) $url .= '&Author='.$u_author;

		// Fetch the XML...
		$snoopy->fetch($url);
		$xml = $snoopy->results;

		// And build an array out of it.
		$data = XML_unserialize($xml);

		/*
		echo '<pre>';
		var_dump($data);
		echo '</pre>';
		*/

		$num_results = $data['ItemSearchResponse']['Items']['TotalResults'];

		if($num_results > 0) {

			// Amazon changes the array structure of its result if there is only one search result returned, so we need to bodge it in that case.
			if($num_results > 1) {
				$items = $data['ItemSearchResponse']['Items']['Item'];
			} else {
				$items = array($data['ItemSearchResponse']['Items']['Item']);
			}

			if(!empty($u_author)) $author = ' by <strong>'.$u_author.'</strong>';

			echo '<p>You searched for <cite>'.$u_title.'</cite>'.$author.'. Amazon returned these results:</p>';

			// Loop through the items that are returned, and display a radio button by each of them.
			foreach($items as $item) {

				$author = (is_array($item['ItemAttributes']['Author'])) ? $item['ItemAttributes']['Author'][0] : $item['ItemAttributes']['Author'];
				$title = $item['ItemAttributes']['Title'];
				$image = (empty($item['MediumImage']['URL'])) ? 'http://robm.me.uk/wp-content/no-image.png' : $item['MediumImage']['URL'];
				$asin = $item['ASIN'];

				// Pack 'em up!
				$details = serialize(compact('author', 'title', 'image', 'asin'));

				echo '
				<form method="post" action="'.$_SERVER['REQUEST_URI'].'" style="border:1px solid #ccc; padding:5px; margin:5px;">

					<input type="hidden" name="amazon_data" value="'.htmlentities($details).'" />

					<img src="'.$image.'" alt="" style="float:left; margin:8px; padding:2px; width:46px; height:70px; border:1px solid #ccc;" />

					<h3>'.$title.'</h3>
					<p>by <strong>'.$author.'</strong></p>

					<p style="clear:left;"><input type="submit" value="Use This Result" /></p>

				</form>
				';
			}

		} else {

			echo '<p>Sorry, but Amazon didn\'t return any results for <cite>'.$u_title.'</cite>'.$author.'.</p>';

		}

	}

	elseif(!empty($_POST['amazon_data'])) {

		$data = unserialize($_POST['amazon_data']);

		$a_author = mysql_real_escape_string($data['author']);
		$a_title = mysql_real_escape_string($data['title']);
		$a_image = mysql_real_escape_string($data['image']);
		$a_asin = mysql_real_escape_string($data['asin']);

		$wpdb->query('
		INSERT INTO '.$table_prefix.'now_reading
		(b_added, b_title, b_author, b_image, b_asin, b_status)
		VALUES(NOW(), "'.$a_title.'", "'.$a_author.'", "'.$a_image.'", "'.$a_asin.'", "unread")
		');

		echo '
		<p>Book added successfully.</p>
		';
	}

	else {

		echo '

		<h3>Add a new book</h3>

		<p>Enter a book\'s title and, optionally, its author and I\'ll try to fetch its information directly from <a href="http://amazon.com">Amazon</a>.</p>

		<form method="post" action="'.$_SERVER['REQUEST_URI'].'">

			<p><label for="title">Title:</label> (required)<br />
			<input type="text" name="u_title" id="title" size="50" /></p>

			<p><label for="title">Author:</label><br />
			<input type="text" name="u_author" id="author" size="50" /></p>

			<p><input type="submit" value="Search" /></p>

		</form>
		';

	}

	echo '
	</div>
	';

}

add_action('admin_menu', 'nr_add_pages');

// This is called from the template file, and displays the book according to the format string.
function nr_get_book($filter_status = 'unread reading read') {

	$filter_status = split(' ', $filter_status);

	foreach($_POST as $key => $val) {
		$_POST[$key] = stripslashes($val);
	}

	global $wpdb, $table_prefix;

	$current_format_string = get_option('nr_current_format_string');
	$read_format_string = get_option('nr_read_format_string');
	$unread_format_string = get_option('nr_unread_format_string');
	$favorites_format_string = get_option('nr_favorites_format_string');

	$num_current = get_option('nr_num_current');
	$num_unread = get_option('nr_num_unread');
	$num_read = get_option('nr_num_read');
	$num_favorites = get_option('nr_num_favorites');

	$current_outer_tag = get_option('nr_current_outer_tag');
	$unread_outer_tag = get_option('nr_unread_outer_tag');
	$read_outer_tag = get_option('nr_read_outer_tag');
	$favorites_outer_tag = get_option('nr_favorites_outer_tag');

	$current_text = get_option('nr_current_text');
	$unread_text = get_option('nr_unread_text');
	$read_text = get_option('nr_read_text');
	$favorites_text = get_option('nr_favorites_text');

	$format_date = get_option('nr_format_date');
	$associate = get_option('nr_associate');
	$domain = get_option('nr_domain');

	$count = $wpdb->get_row('
	SELECT COUNT(*) AS count
	FROM '.$table_prefix.'now_reading
	');

	if($count->count > 0) {

		$variables = array(
			'%title%',
			'%author%',
			'%image%',
			'%date%',
			'%link%',
			'%review%'
		);

		if (in_array('unread', $filter_status)) {
			$unread = $wpdb->get_results('
			SELECT b_title AS title, b_author AS author, b_image AS image, DATE_FORMAT(b_added, "'.$format_date.'") AS date, b_asin AS asin, COUNT(*) AS count, b_review AS review
			FROM '.$table_prefix.'now_reading
			WHERE b_status = "unread"
			GROUP BY b_id
			ORDER BY b_added DESC
			LIMIT '.$num_unread.'
			');
		}

		if (in_array('reading', $filter_status)) {
			$current = $wpdb->get_results('
			SELECT b_title AS title, b_author AS author, b_image AS image, DATE_FORMAT(b_added, "'.$format_date.'") AS date, b_asin AS asin, COUNT(*) AS count, b_review AS review
			FROM '.$table_prefix.'now_reading
			WHERE b_status = "reading"
			GROUP BY b_id
			ORDER BY b_added DESC
			LIMIT '.$num_current.'
			');
		}

		if (in_array('read', $filter_status)) {
			$read = $wpdb->get_results('
			SELECT b_title AS title, b_author AS author, b_image AS image, DATE_FORMAT(b_added, "'.$format_date.'") AS date, b_asin AS asin, COUNT(*) AS count, b_review AS review
			FROM '.$table_prefix.'now_reading
			WHERE b_status = "read"
			GROUP BY b_id
			ORDER BY b_added DESC
			LIMIT '.$num_read.'
			');
		}

		if (in_array('favorites', $filter_status)) {
			$favorites = $wpdb->get_results('
			SELECT b_title AS title, b_author AS author, b_image AS image, DATE_FORMAT(b_added, "'.$format_date.'") AS date, b_asin AS asin, COUNT(*) AS count, b_review AS review
			FROM '.$table_prefix.'now_reading
			WHERE b_status = "favorites"
			GROUP BY b_id
			ORDER BY b_added DESC
			LIMIT '.$num_favorites.'
			');
		}

		if($unread[0]->count && in_array('unread', $filter_status)) {
			echo "
			$unread_text
				<$unread_outer_tag>
			";

			foreach($unread as $book) {

				$replacements = array(
					$book->title,
					$book->author,
					$book->image,
					$book->date,
					"http://amazon$domain/exec/obidos/tg/detail/-/{$book->asin}/$associate",
					$book->review
				);

				echo str_replace($variables, $replacements, $unread_format_string);

			}

			echo "
				</$unread_outer_tag>
			";
		}

		if($current[0]->count && in_array('reading', $filter_status)) {
			echo "
			$current_text
				<$current_outer_tag>
			";

			foreach($current as $book) {

				$replacements = array(
					$book->title,
					$book->author,
					$book->image,
					$book->date,
					"http://amazon$domain/exec/obidos/tg/detail/-/{$book->asin}/$associate",
					$book->review
				);

				echo str_replace($variables, $replacements, $current_format_string);

			}

			echo "
				</$read_outer_tag>
			";
		}

		if($read[0]->count && in_array('read', $filter_status)) {
			echo "
			$read_text
				<$read_outer_tag>
			";

			foreach($read as $book) {

				$replacements = array(
					$book->title,
					$book->author,
					$book->image,
					$book->date,
					"http://amazon$domain/exec/obidos/tg/detail/-/{$book->asin}/$associate",
					$book->review
				);

				echo str_replace($variables, $replacements, $read_format_string);

			}

			echo "
				</$read_outer_tag>
			";
		}

		if($favorites[0]->count && in_array('favorites', $filter_status)) {
			echo "
			$favorites_text
				<$favorites_outer_tag>
			";

			foreach($favorites as $book) {

				$replacements = array(
					$book->title,
					$book->author,
					$book->image,
					$book->date,
					"http://amazon$domain/exec/obidos/tg/detail/-/{$book->asin}/$associate",
					$book->review
				);

				echo str_replace($variables, $replacements, $favorites_format_string);

			}

			echo "
				</$favorites_outer_tag>
			";
		}

	}

}

###################################################################################
#
# XML Library, by Keith Devens, version 1.2b
# http://keithdevens.com/software/phpxml
#
# This code is Open Source, released under terms similar to the Artistic License.
# Read the license at http://keithdevens.com/software/license
#
###################################################################################

###################################################################################
# XML_unserialize: takes raw XML as a parameter (a string)
# and returns an equivalent PHP data structure
###################################################################################
function & XML_unserialize(&$xml){
	$xml_parser = &new XML();
	$data = &$xml_parser->parse($xml);
	$xml_parser->destruct();
	return $data;
}
###################################################################################
# XML_serialize: serializes any PHP data structure into XML
# Takes one parameter: the data to serialize. Must be an array.
###################################################################################
function & XML_serialize(&$data, $level = 0, $prior_key = NULL){
	if($level == 0){ ob_start(); echo '<?xml version="1.0" ?>',"\n"; }
	while(list($key, $value) = each($data))
		if(!strpos($key, ' attr')) #if it's not an attribute
			#we don't treat attributes by themselves, so for an empty element
			# that has attributes you still need to set the element to NULL

			if(is_array($value) and array_key_exists(0, $value)){
				XML_serialize($value, $level, $key);
			}else{
				$tag = $prior_key ? $prior_key : $key;
				echo str_repeat("\t", $level),'<',$tag;
				if(array_key_exists("$key attr", $data)){ #if there's an attribute for this element
					while(list($attr_name, $attr_value) = each($data["$key attr"]))
						echo ' ',$attr_name,'="',htmlspecialchars($attr_value),'"';
					reset($data["$key attr"]);
				}

				if(is_null($value)) echo " />\n";
				elseif(!is_array($value)) echo '>',htmlspecialchars($value),"</$tag>\n";
				else echo ">\n",XML_serialize($value, $level+1),str_repeat("\t", $level),"</$tag>\n";
			}
	reset($data);
	if($level == 0){ $str = &ob_get_contents(); ob_end_clean(); return $str; }
}
###################################################################################
# XML class: utility class to be used with PHP's XML handling functions
###################################################################################
class XML{
	var $parser;   #a reference to the XML parser
	var $document; #the entire XML structure built up so far
	var $parent;   #a pointer to the current parent - the parent will be an array
	var $stack;    #a stack of the most recent parent at each nesting level
	var $last_opened_tag; #keeps track of the last tag opened.

	function XML(){
 		$this->parser = &xml_parser_create();
		xml_parser_set_option(&$this->parser, XML_OPTION_CASE_FOLDING, false);
		xml_set_object(&$this->parser, &$this);
		xml_set_element_handler(&$this->parser, 'open','close');
		xml_set_character_data_handler(&$this->parser, 'data');
	}
	function destruct(){ xml_parser_free(&$this->parser); }
	function & parse(&$data){
		$this->document = array();
		$this->stack    = array();
		$this->parent   = &$this->document;
		return xml_parse(&$this->parser, &$data, true) ? $this->document : NULL;
	}
	function open(&$parser, $tag, $attributes){
		$this->data = ''; #stores temporary cdata
		$this->last_opened_tag = $tag;
		if(is_array($this->parent) and array_key_exists($tag,$this->parent)){ #if you've seen this tag before
			if(is_array($this->parent[$tag]) and array_key_exists(0,$this->parent[$tag])){ #if the keys are numeric
				#this is the third or later instance of $tag we've come across
				$key = count_numeric_items($this->parent[$tag]);
			}else{
				#this is the second instance of $tag that we've seen. shift around
				if(array_key_exists("$tag attr",$this->parent)){
					$arr = array('0 attr'=>&$this->parent["$tag attr"], &$this->parent[$tag]);
					unset($this->parent["$tag attr"]);
				}else{
					$arr = array(&$this->parent[$tag]);
				}
				$this->parent[$tag] = &$arr;
				$key = 1;
			}
			$this->parent = &$this->parent[$tag];
		}else{
			$key = $tag;
		}
		if($attributes) $this->parent["$key attr"] = $attributes;
		$this->parent  = &$this->parent[$key];
		$this->stack[] = &$this->parent;
	}
	function data(&$parser, $data){
		if($this->last_opened_tag != NULL) #you don't need to store whitespace in between tags
			$this->data .= $data;
	}
	function close(&$parser, $tag){
		if($this->last_opened_tag == $tag){
			$this->parent = $this->data;
			$this->last_opened_tag = NULL;
		}
		array_pop($this->stack);
		if($this->stack) $this->parent = &$this->stack[count($this->stack)-1];
	}
}
function count_numeric_items(&$array){
	return is_array($array) ? count(array_filter(array_keys($array), 'is_numeric')) : 0;
}

?>
